#!/usr/bin/env python
# -*- mode:Python; tab-width: 4 -*-
"""
Big hack for Etxebide. Goes through the html cloning the images
and non-html files into relative images and datos subdirectories.
"""

import re
import os
import shutil
import html_parser
import cms_freeze
import css_parser


__author__ = "Grzegorz Adam Hankiewicz <gradha@efaber.net>"
__date__ = "Jul 2003"
__version__ = "$Revision: 1.2 $"
__credits__ = ""


html_extension = ".htm"
non_graphic_file = re.compile(r"/files/|\.swf|\.css")
ignore_paths = re.compile(r"images/|datos/")
delete_files = {}


def main(directory, extension):
    """Entry point of the appliaction."""
    html_parser.watched_tags["param"] = "value"
    for file, dummy in cms_freeze.walk_local_files(extension, directory):
        process_html_links(file, directory)
    for file, dummy in cms_freeze.walk_local_files(".css", directory):
        process_css_links(file, directory)

    for file in delete_files.iterkeys():
        try: os.unlink(file)
        except OSError: pass
    for remove in ["graphics", os.path.join("etx", "i"), "files"]:
        try: shutil.rmtree(os.path.join(directory, remove), 1)
        except OSError: pass
    print "Done"


def bastardize_relative_link(path):
    """Removes '../' and changes other slashes to underscores."""
    return path.replace("../", "").replace("/", "_")


def process_css_links(filename, root_directory):
    """Makes all images and data links relative to local folder."""
    print "Searching non relative links in", filename
    input = file(filename, "rt")
    l = css_parser.Link_extractor(input.read())
    input.close()
    substitutions = {}
    for link in l.get_links():
        if ignore_paths.match(link) or link.find(".css") > 0:
            continue
        directory = os.path.realpath(os.path.dirname(filename))
        full_path = os.path.realpath(os.path.join(directory, link))
        if not os.path.isfile(full_path):
            continue

        final_directory = os.path.join(root_directory, "images")
        try:
            os.mkdir(final_directory)
        except OSError: pass

        final_filename = bastardize_relative_link(link)
        final_filename = os.path.join(final_directory, final_filename)

        if not os.path.isfile(final_filename):
            shutil.copyfile(full_path, final_filename)
        if os.path.samefile(full_path, final_filename):
            continue
        delete_files[full_path] = 1
            
        relative_link = html_parser.relative_path("%s/" % directory, final_filename)

        print "\tde `%s' a `%s'" % (link, relative_link)
        substitutions[link] = final_filename

    # ok, make changes if links were found
    if not substitutions:
        return

    t = css_parser.Link_transformer(filename, "", substitutions, 0, 1)
    output = file(filename, "wb")
    output.write(t.get_data())
    output.close()
    

def process_html_links(filename, root_directory):
    """Makes all images and data links relative to local folder."""
    print "Searching non relative links in", filename
    input = file(filename, "rb")
    data = input.read()
    input.close()
    substitutions = {}
    l = html_parser.Link_extractor(data, 0)
    for link in l.get_links():
        if ignore_paths.match(link) or link.find(".css") > 0:
            continue
        directory = os.path.realpath(os.path.dirname(filename))
        full_path = os.path.realpath(os.path.join(directory, link))
        if not os.path.isfile(full_path):
            continue
        if link.find(html_extension) > 0:
            continue

        if non_graphic_file.search(link):
            final_directory = os.path.join(root_directory, "datos")
        #elif link.find(".css") > 0:
        #    final_directory = directory[:]
        else:
            final_directory = os.path.join(root_directory, "images")
        try:
            os.mkdir(final_directory)
        except OSError: pass

        final_filename = bastardize_relative_link(link)
        final_filename = os.path.join(final_directory, final_filename)

        if not os.path.isfile(final_filename):
            shutil.copyfile(full_path, final_filename)
        if os.path.samefile(full_path, final_filename):
            continue
        delete_files[full_path] = 1
            
        relative_link = html_parser.relative_path("%s/" % directory, final_filename)

        print "\tde `%s' a `%s'" % (link, relative_link)
        substitutions[link] = final_filename

    # ok, make changes if links were found
    if not substitutions:
        return

    t = html_parser.Link_transformer(filename, "", substitutions, 0, 1)
    output = file(filename, "wb")
    output.write(t.get_data())
    output.close()


if __name__ == "__main__":
    import sys
    if "run" in sys.argv[1:]:
        main("etxebide", html_extension)
    else:
        print """
Big hack for Etxebide. Goes through the html cloning the images
and non-html files into relative images and datos subdirectories.
All the useful values are hardcoded. Usually you will be running
this after freeze_etxebide.py.

To run this script for real, you have to pass the parameter
"run". Otherwise only this boring screen is printed.  WARNING:
RUNNING THIS SCRIPT MORE THAN ONCE WILL SPOIL THE RESULT!
"""
